@@ -284,9 +284,11 @@ struct tcb {
 #define TCB_FOLLOWFORK	00400	/* Process should have forks followed */
 #define TCB_REPRINT	01000	/* We should reprint this syscall on exit */
 #ifdef LINUX
-#if defined(ALPHA) || defined(SPARC) || defined(POWERPC) || defined(HPPA)
+#if defined(ALPHA) || defined(SPARC) || defined(POWERPC) || defined(HPPA) || defined(IA64)
+   /* When execve'ing an ELF binary, the kernel sends an extra SIGTRAP
+      which we need to ignore.  */
 #define TCB_WAITEXECVE	02000	/* ignore SIGTRAP after exceve */
-#endif /* ALPHA */
+#endif /* ALPHA et al. */
 #endif /* LINUX */
 
 /* qualifier flags */
