@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: mem.c,v 1.11 1999/12/23 14:20:15 wichert Exp $
+ *	$Id: mem.c,v 1.20 2001/03/08 14:40:06 hughesj Exp $
  */
 
 #include "defs.h"
@@ -41,6 +41,13 @@
 #include <asm/ldt.h>
 #endif
 
+#ifdef HAVE_LONG_LONG_OFF_T
+/*
+ * Ugly hacks for systems that have a long long off_t
+ */
+#define sys_mmap64	sys_mmap
+#endif
+
 int
 sys_brk(tcp)
 struct tcb *tcp;
@@ -133,10 +140,27 @@ static struct xlat mmap_flags[] = {
 #endif
 #ifdef MAP_LOCKED
 	{ MAP_LOCKED,"MAP_LOCKED"},
+#endif
+	/* FreeBSD ones */
+#ifdef MAP_ANON
+	{ MAP_ANON,		"MAP_ANON"	},
+#endif
+#ifdef MAP_HASSEMAPHORE
+	{ MAP_HASSEMAPHORE,	"MAP_HASSEMAPHORE"	},
+#endif
+#ifdef MAP_STACK
+	{ MAP_STACK,		"MAP_STACK"	},
+#endif
+#ifdef MAP_NOSYNC
+	{ MAP_NOSYNC,		"MAP_NOSYNC"	},
+#endif
+#ifdef MAP_NOCORE
+	{ MAP_NOCORE,		"MAP_NOCORE"	},
 #endif
 	{ 0,		NULL		},
 };
 
+#if !HAVE_LONG_LONG_OFF_T
 static
 int
 print_mmap(tcp,u_arg)
@@ -155,8 +179,12 @@ long *u_arg;
 		printflags(mmap_prot, u_arg[2]);
 		tprintf(", ");
 		/* flags */
+#ifdef MAP_TYPE
 		printxval(mmap_flags, u_arg[3] & MAP_TYPE, "MAP_???");
 		addflags(mmap_flags, u_arg[3] & ~MAP_TYPE);
+#else
+		printflags(mmap_flags, u_arg[3]);
+#endif
 		/* fd */
 		tprintf(", %ld, ", u_arg[4]);
 		/* offset */
@@ -171,8 +199,27 @@ struct tcb *tcp;
 {
     long u_arg[6];
 
+#if	defined(IA64)
+    int i, v;
+    /*
+     *  IA64 processes never call this routine, they only use the
+     *  new `sys_mmap' interface.  This code converts the integer
+     *  arguments that the IA32 process pushed onto the stack into
+     *  longs.
+     *
+     *  Note that addresses with bit 31 set will be sign extended.
+     *  Fortunately, those addresses are not currently being generated
+     *  for IA32 processes so it's not a problem.
+     */
+    for (i = 0; i < 6; i++)
+	if (umove(tcp, tcp->u_arg[0] + (i * sizeof(int)), &v) == -1)
+		return 0;
+	else
+		u_arg[i] = v;
+#else	// defined(IA64)
     if (umoven(tcp, tcp->u_arg[0], sizeof u_arg, (char *) u_arg) == -1)
 	    return 0;
+#endif	// defined(IA64)
     return print_mmap(tcp, u_arg);
    
 }
@@ -184,7 +231,57 @@ struct tcb *tcp;
 {
     return print_mmap(tcp, tcp->u_arg);
 }
+#endif /* !HAVE_LONG_LONG_OFF_T */
 
+#if _LFS64_LARGEFILE || HAVE_LONG_LONG_OFF_T
+int
+sys_mmap64(tcp)
+struct tcb *tcp;
+{
+#ifdef linux
+#ifdef ALPHA
+	long *u_arg = tcp->u_arg;
+#else /* !ALPHA */
+	long u_arg[7];
+#endif /* !ALPHA */
+#else /* !linux */
+	long *u_arg = tcp->u_arg;
+#endif /* !linux */
+
+	if (entering(tcp)) {
+#ifdef linux
+#ifndef ALPHA
+		if (umoven(tcp, tcp->u_arg[0], sizeof u_arg,
+				(char *) u_arg) == -1)
+			return 0;
+#endif /* ALPHA */
+#endif /* linux */
+		ALIGN64 (tcp, 5);	/* FreeBSD wierdies */
+
+		/* addr */
+		tprintf("%#lx, ", u_arg[0]);
+		/* len */
+		tprintf("%lu, ", u_arg[1]);
+		/* prot */
+		printflags(mmap_prot, u_arg[2]);
+		tprintf(", ");
+		/* flags */
+#ifdef MAP_TYPE
+		printxval(mmap_flags, u_arg[3] & MAP_TYPE, "MAP_???");
+		addflags(mmap_flags, u_arg[3] & ~MAP_TYPE);
+#else
+		printflags(mmap_flags, u_arg[3]);
+#endif
+		/* fd */
+		tprintf(", %ld, ", u_arg[4]);
+		/* offset */
+		tprintf("%#llx", LONG_LONG(u_arg[5], u_arg[6]));
+	}
+	return RVAL_HEX;
+}
+#endif
+
+ 
 int
 sys_munmap(tcp)
 struct tcb *tcp;
@@ -212,7 +309,8 @@ struct tcb *tcp;
 #ifdef LINUX
 
 static struct xlat mremap_flags[] = {
-	{ MREMAP_MAYMOVE, "MREMAP_MAYMOVE" },
+	{ MREMAP_MAYMOVE,	"MREMAP_MAYMOVE"	},
+	{ 0,			NULL			}
 };
 
 int
@@ -227,6 +325,59 @@ struct tcb *tcp;
 	return RVAL_HEX;
 }
 
+static struct xlat madvise_flags[] = {
+#ifdef MADV_NORMAL
+	{ MADV_NORMAL,		"MADV_NORMAL" },
+#endif
+#ifdef MADZV_RANDOM
+	{ MADV_RANDOM,		"MADV_RANDOM" },
+#endif
+#ifdef MADV_SEQUENTIAL
+	{ MADV_SEQUENTIAL,	"MADV_SEQUENTIAL" },
+#endif
+#ifdef MADV_WILLNEED
+	{ MADV_WILLNEED,	"MADV_WILLNEED" },
+#endif
+#ifdef MADV_DONTNED
+	{ MADV_DONTNEED,	"MADV_DONTNEED" },
+#endif
+	{ 0,			NULL },
+};
+
+
+int
+sys_madvise(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%#lx, %lu, ", tcp->u_arg[0], tcp->u_arg[1]);
+		printflags(madvise_flags, tcp->u_arg[2]);
+	}
+	return 0;
+}
+
+
+static struct xlat mlockall_flags[] = {
+#ifdef MCL_CURRENT
+	{ MCL_CURRENT,	"MCL_CURRENT" },
+#endif
+#ifdef MCL_FUTURE
+	{ MCL_FUTURE,	"MCL_FUTURE" },
+#endif
+	{ 0,		NULL}
+};
+
+int
+sys_mlockall(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printflags(mlockall_flags, tcp->u_arg[0]);
+	}
+	return 0;
+}
+
+
 #endif /* LINUX */
 
 #ifdef MS_ASYNC
