@@ -1198,9 +1198,9 @@ struct tcb *tcp;
 		}
 	}
 #ifdef LINUX
-#if defined(ALPHA) || defined(SPARC) || defined(POWERPC) || defined(HPPA)
+#if defined(ALPHA) || defined(SPARC) || defined(POWERPC) || defined(HPPA) || defined(IA64)
 	tcp->flags |= TCB_WAITEXECVE;
-#endif /* ALPHA || SPARC || POWERPC */
+#endif /* ALPHA || SPARC || POWERPC || HPPA || IA64 */
 #endif /* LINUX */
 	return 0;
 }
