@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: resource.c,v 1.3 1999/12/23 14:20:15 wichert Exp $
+ *	$Id: resource.c,v 1.7 2001/03/08 13:59:00 hughesj Exp $
  */
 
 #include "defs.h"
@@ -36,15 +36,29 @@
 #ifdef LINUX
 #include <sys/times.h>
 #include <linux/kernel.h>
+#include <sys/quota.h>
 #endif /* LINUX */
 #ifdef SUNOS4
 #include <ufs/quota.h>
 #endif /* SUNOS4 */
-#ifdef SVR4
+#if defined(SVR4) || defined(FREEBSD)
 #include <sys/times.h>
 #include <sys/time.h>
 #endif
 
+#if HAVE_LONG_LONG_RLIM_T
+/*
+ * Hacks for systems that have a long long rlim_t
+ */
+
+#define rlimit64 rlimit			/* Ugly hack */
+#define rlim64_t rlim_t			/* Ugly hack */
+#define RLIM64_INFINITY RLIM_INFINITY	/* You guessed it */
+
+#define sys_getrlimit64	sys_getrlimit
+#define sys_setrlimit64	sys_setrlimit
+#endif
+
 static struct xlat resources[] = {
 #ifdef RLIMIT_CPU
 	{ RLIMIT_CPU,	"RLIMIT_CPU"	},
@@ -64,9 +78,15 @@ static struct xlat resources[] = {
 #ifdef RLIMIT_RSS
 	{ RLIMIT_RSS,	"RLIMIT_RSS"	},
 #endif
+#ifdef RLIMIT_NPROC
+	{ RLIMIT_NPROC,"RLIMIT_NPROC"	},
+#endif
 #ifdef RLIMIT_NOFILE
 	{ RLIMIT_NOFILE,"RLIMIT_NOFILE"	},
 #endif
+#ifdef RLIMIT_MEMLOCK
+	{ RLIMIT_MEMLOCK,	"RLIMIT_MEMLOCK"	},
+#endif
 #ifdef RLIMIT_VMEM
 	{ RLIMIT_VMEM,	"RLIMIT_VMEM"	},
 #endif
@@ -76,6 +96,7 @@ static struct xlat resources[] = {
 	{ 0,		NULL		},
 };
 
+#if !HAVE_LONG_LONG_RLIM_T
 static char *
 sprintrlim(lim)
 long lim;
@@ -134,12 +155,77 @@ struct tcb *tcp;
 	}
 	return 0;
 }
+#endif /* !HAVE_LONG_LONG_RLIM_T */
+
+#if _LFS64_LARGEFILE || HAVE_LONG_LONG_RLIM_T
+static char *
+sprintrlim64(lim)
+rlim64_t lim;
+{
+	static char buf[64];
+
+	if (lim == RLIM64_INFINITY)
+		sprintf(buf, "RLIM64_INFINITY");
+	else if (lim > 1024 && lim%1024 == 0)
+		sprintf(buf, "%lld*1024", lim/1024);
+	else
+		sprintf(buf, "%lld", lim);
+	return buf;
+}
+
+int
+sys_getrlimit64(tcp)
+struct tcb *tcp;
+{
+	struct rlimit64 rlim;
+
+	if (entering(tcp)) {
+		printxval(resources, tcp->u_arg[0], "RLIMIT_???");
+		tprintf(", ");
+	}
+	else {
+		if (syserror(tcp) || !verbose(tcp))
+			tprintf("%#lx", tcp->u_arg[1]);
+		else if (umove(tcp, tcp->u_arg[1], &rlim) < 0)
+			tprintf("{...}");
+		else {
+			tprintf("{rlim_cur=%s,", sprintrlim64(rlim.rlim_cur));
+			tprintf(" rlim_max=%s}", sprintrlim64(rlim.rlim_max));
+		}
+	}
+	return 0;
+}
+
+int
+sys_setrlimit64(tcp)
+struct tcb *tcp;
+{
+	struct rlimit64 rlim;
+
+	if (entering(tcp)) {
+		printxval(resources, tcp->u_arg[0], "RLIMIT_???");
+		tprintf(", ");
+		if (!verbose(tcp))
+			tprintf("%#lx", tcp->u_arg[1]);
+		else if (umove(tcp, tcp->u_arg[1], &rlim) < 0)
+			tprintf("{...}");
+		else {
+			tprintf("{rlim_cur=%s,", sprintrlim64(rlim.rlim_cur));
+			tprintf(" rlim_max=%s}", sprintrlim64(rlim.rlim_max));
+		}
+	}
+	return 0;
+}
+#endif /* _LFS64_LARGEFILES || HAVE_LONG_LONG_RLIM_T */
 
 #ifndef SVR4
 
 static struct xlat usagewho[] = {
 	{ RUSAGE_SELF,		"RUSAGE_SELF"		},
 	{ RUSAGE_CHILDREN,	"RUSAGE_CHILDREN"	},
+#ifdef RUSAGE_BOTH
+	{ RUSAGE_BOTH,		"RUSAGE_BOTH"		},
+#endif
 	{ 0,			NULL			},
 };
 
@@ -367,14 +453,85 @@ struct tcb *tcp;
 
 #endif /* !SUNOS4 */
 
-#ifdef SUNOS4
+#ifdef LINUX
 
 static struct xlat quotacmds[] = {
 	{ Q_QUOTAON,	"Q_QUOTAON"	},
 	{ Q_QUOTAOFF,	"Q_QUOTAOFF"	},
 	{ Q_GETQUOTA,	"Q_GETQUOTA"	},
 	{ Q_SETQUOTA,	"Q_SETQUOTA"	},
+	{ Q_SETUSE,	"Q_SETUSE"	},
+	{ Q_SYNC,	"Q_SYNC"	},
 	{ Q_SETQLIM,	"Q_SETQLIM"	},
+	{ Q_GETSTATS,	"Q_GETSTATS"	},
+	{ Q_RSQUASH,	"Q_RSQUASH"	},
+	{ 0,		NULL		},
+};
+
+static struct xlat quotatypes[] = {
+	{ USRQUOTA,	"USRQUOTA"	},
+	{ GRPQUOTA,	"GRPQUOTA"	},
+	{ 0,		NULL		},
+};
+
+int
+sys_quotactl(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printxval(quotacmds, tcp->u_arg[0] >> SUBCMDSHIFT, "Q_???");
+		tprintf("|");
+		printxval(quotatypes, tcp->u_arg[0] & SUBCMDMASK, "???QUOTA");
+		tprintf(", ");
+		printstr(tcp, tcp->u_arg[1], -1);
+		tprintf(", %lu, ", tcp->u_arg[2]);
+	}
+	else {
+		struct dqblk dq;
+
+		if (!tcp->u_arg[3])
+			tprintf("NULL");
+		else if (!verbose(tcp))
+			tprintf("%#lx", tcp->u_arg[3]);
+                else if (umoven(tcp, tcp->u_arg[3], sizeof(struct dqblk),   
+                    (char *) &dq) < 0)
+                        tprintf("???");
+		else {
+                        tprintf("{");
+			tprintf("%u, ", dq.dqb_bhardlimit);
+			tprintf("%u, ", dq.dqb_bsoftlimit);
+			tprintf("%u, ", dq.dqb_curblocks);
+			tprintf("%u, ", dq.dqb_ihardlimit);
+			tprintf("%u, ", dq.dqb_isoftlimit);
+			tprintf("%u, ", dq.dqb_curinodes);
+			tprintf("%lu, ", dq.dqb_btime);
+			tprintf("%lu", dq.dqb_itime);
+                        tprintf("}");
+		}
+
+	}
+	return 0;
+}
+
+#endif /* Linux */
+
+#if defined(SUNOS4) || defined(FREEBSD)
+
+#ifdef FREEBSD
+#include <ufs/ufs/quota.h>
+#endif
+
+static struct xlat quotacmds[] = {
+	{ Q_QUOTAON,	"Q_QUOTAON"	},
+	{ Q_QUOTAOFF,	"Q_QUOTAOFF"	},
+	{ Q_GETQUOTA,	"Q_GETQUOTA"	},
+	{ Q_SETQUOTA,	"Q_SETQUOTA"	},
+#ifdef Q_SETQLIM
+	{ Q_SETQLIM,	"Q_SETQLIM"	},
+#endif
+#ifdef Q_SETUSE
+	{ Q_SETUSE,	"Q_SETUSE"	},
+#endif
 	{ Q_SYNC,	"Q_SYNC"	},
 	{ 0,		NULL		},
 };
@@ -385,12 +542,19 @@ struct tcb *tcp;
 {
 	/* fourth arg (addr) not interpreted here */
 	if (entering(tcp)) {
+#ifdef SUNOS4
 		printxval(quotacmds, tcp->u_arg[0], "Q_???");
 		tprintf(", ");
 		printstr(tcp, tcp->u_arg[1], -1);
+#endif
+#ifdef FREEBSD
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+		printxval(quotacmds, tcp->u_arg[1], "Q_???");
+#endif		
 		tprintf(", %lu, %#lx", tcp->u_arg[2], tcp->u_arg[3]);
 	}
 	return 0;
 }
 
-#endif /* SUNOS4 */
+#endif /* SUNOS4 || FREEBSD */
